@@ -1,13 +1,10 @@
 #' Build a Markov Model
 #'
-#' Function \code{build_mm} is a shortcut for constructing a Markov model as a restricted 
-#'   case of an \code{hmm} object.
+#' Function \code{build_mm} builds and automatically estimates a Markov model. It is also a shortcut for 
+#' constructing a Markov model as a restricted case of an \code{hmm} object.
 #' @export
 #' @param observations An \code{stslist} object (see \code{\link[TraMineR]{seqdef}}) containing
 #' the sequences.
-#' @param transition_probs A matrix of transition probabilities. Transition probabilities should follow the
-#' ordering of the alphabet of observations (\code{alphabet(observations)}, returned as \code{state_names}).
-#' @param initial_probs A vector of initial state probabilities.
 #' @return Object of class \code{hmm} with following elements:
 #' \describe{
 #'    \item{\code{observations}}{State sequence object or a list of such containing the data.}
@@ -24,6 +21,13 @@
 #'    \item{\code{n_channels}}{Number of channels.}
 #'}
 #'
+#' @details Unlike the other build functions in \code{seqHMM}, the \code{build_mm} function
+#' automatically estimates the model parameters. As initial and transition probabilities can be
+#' directly estimated from the observed initial state probabilities and transition counts, there
+#' is no need for starting values or further estimation with the \code{\link{fit_model}} function.
+#'
+#' @seealso \code{\link{plot.hmm}} for plotting the model.
+#' 
 #' @examples
 #' # Construct sequence data
 #' data("mvad", package = "TraMineR")
@@ -39,50 +43,26 @@
 #' # Define a color palette for the sequence data
 #' attr(mvad_seq, "cpal") <- colorpalette[[6]]
 #'
-#' # Initialize a Markov model
-#' init_mm_mvad <- build_mm(observations = mvad_seq,
-#'   transition_probs = simulate_transition_probs(6),
-#'   initial_probs = rep(1/6,6))
+#' # Estimate the Markov model
+#' mm_mvad <- build_mm(observations = mvad_seq)
 #'
-#' # Fit the model
-#' mm_mvad <- fit_model(init_mm_mvad)
-#'
-#' @seealso \code{\link{fit_model}} for estimating model parameters.
 
-build_mm <- function(observations, transition_probs, initial_probs){
+build_mm <- function(observations){
 
   if(!inherits(observations, "stslist")){
     stop("The build_mm function can only be used for single-channel sequence data (as an stslist object). Use the mc_to_sc_data function to convert multiple stslist into single-channel state sequences.")
   }
 
-  n_sequences<-nrow(observations)
-  length_of_sequences<-ncol(observations)
-  state_names<-alphabet(observations)
-  n_states<-length(state_names)
-
-  if (!is.matrix(transition_probs)) {
-    stop(paste("Object provided for transition_probs is not a matrix."))
-  }
-  if (!is.vector(initial_probs)) {
-    stop(paste("Object provided for initial_probs is not a vector."))
-  }
-
-  if(dim(transition_probs)[1]!=dim(transition_probs)[2])
-    stop("Transition_probs must be a square matrix.")
-
-  if (n_states != nrow(transition_probs))
-    stop("Number of rows in transition_probs is not equal to the number of distinct symbols in observations.")
-
-  if (length(initial_probs) != n_states){
-    stop(paste("Length of initial_probs is not equal to the number of states."))
-  }
-
-  if(!isTRUE(all.equal(rowSums(transition_probs),rep(1,dim(transition_probs)[1]),check.attributes=FALSE)))
-    stop("Transition probabilities in transition_probs do not sum to one.")
-
-  dimnames(transition_probs) <- list(from=state_names,to=state_names)
-
+  n_sequences <- nrow(observations)
+  length_of_sequences <- ncol(observations)
+  state_names <- alphabet(observations)
+  n_states <- length(state_names)
+  
+  transition_probs <- suppressMessages(TraMineR::seqtrate(observations))
+  
+  initial_probs <- suppressMessages(TraMineR::seqstatf(seqdef(observations[, 1], alphabet = state_names)))[, 2]
 
+  dimnames(transition_probs) <- list(from = state_names, to = state_names)
 
   names(initial_probs) <- state_names
 
@@ -93,17 +73,18 @@ build_mm <- function(observations, transition_probs, initial_probs){
     emission_probs <- diag(n_states)
     dimnames(emission_probs) <- dimnames(transition_probs)
 
-    model <- structure(list(observations=observations,transition_probs=transition_probs,
+    model <- structure(list(observations=observations,
+                            transition_probs=transition_probs,
       emission_probs=emission_probs,initial_probs=initial_probs,
       state_names=state_names,
-      symbol_names=state_names,channel_names=NULL,
+      symbol_names=state_names, channel_names=NULL,
       length_of_sequences=length_of_sequences,
       n_sequences=n_sequences,
       n_symbols=n_states,n_states=n_states,
       n_channels=1), class = "hmm",
       nobs = nobs,
       df = sum(initial_probs > 0) - 1 + sum(transition_probs > 0) - n_states,
-      type = "mmm")
+      type = "mm")
 
   model
 }
