@@ -1,15 +1,16 @@
 // log-likelihood of MHMM using log-space
-#include "seqHMM.h"
+#include <RcppArmadillo.h>
+#include "reparma.h"
 
 // [[Rcpp::export]]
 
-NumericVector logLikMixHMM(const arma::mat& transition, const arma::cube& emission,
+Rcpp::NumericVector logLikMixHMM(const arma::mat& transition, const arma::cube& emission,
   const arma::vec& init, const arma::ucube& obs, const arma::mat& coef, const arma::mat& X,
   const arma::uvec& numberOfStates, unsigned int threads) {
   
   arma::mat weights = exp(X * coef).t();
   if (!weights.is_finite()) {
-    return wrap(-arma::datum::inf);
+    return Rcpp::wrap(-arma::datum::inf);
   }
   weights.each_row() /= sum(weights, 0);
   
@@ -39,5 +40,5 @@ NumericVector logLikMixHMM(const arma::mat& transition, const arma::cube& emissi
         alpha /= tmp;
       }
     }
-    return wrap(ll);
+    return Rcpp::wrap(ll);
 }
