@@ -1,9 +1,9 @@
 // Forward-backward algorithm for non-mixture hidden Markov models using log-space
 
-#include "seqHMM.h"
+#include "log_forward_backward.h"
 // [[Rcpp::export]]
 
-List log_forwardbackward(const arma::mat& transition_, const arma::cube& emission_, 
+Rcpp::List log_forwardbackward(const arma::mat& transition_, const arma::cube& emission_, 
   const arma::vec& init_, const arma::ucube& obs, bool forwardonly, unsigned int threads) {
 
   arma::vec init = log(init_);
@@ -15,11 +15,11 @@ List log_forwardbackward(const arma::mat& transition_, const arma::cube& emissio
   log_internalForward(transition, emission, init, obs, alpha, threads);
 
   if (forwardonly) {
-    return List::create(Named("forward_probs") = wrap(alpha));
+    return Rcpp::List::create(Rcpp::Named("forward_probs") = Rcpp::wrap(alpha));
   } else {
     arma::cube beta(emission.n_rows, obs.n_cols, obs.n_slices); //m,n,k
     log_internalBackward(transition, emission, obs, beta, threads);
-    return List::create(Named("forward_probs") = wrap(alpha), Named("backward_probs") = wrap(beta));
+    return Rcpp::List::create(Rcpp::Named("forward_probs") = Rcpp::wrap(alpha), Rcpp::Named("backward_probs") = Rcpp::wrap(beta));
   }
 
 }
